@@ -19,14 +19,27 @@
 
 #include "config.h"
 
+#include <math.h>
+
+#include <glib/gi18n.h>
+
 #include <libtracker-common/tracker-dbus.h>
 #include <libtracker-common/tracker-type-utils.h>
 
 #include "tracker-marshal.h"
 #include "tracker-miner-object.h"
 #include "tracker-miner-dbus.h"
-#include "tracker-miner-glue.h"
-#include "tracker-dbus.h"
+
+/* Here we use ceil() to eliminate decimal points beyond what we're
+ * interested in, which is 2 decimal places for the progress. The
+ * ceil() call will also round up the last decimal place.
+ *
+ * The 0.49 value is used for rounding correctness, because ceil()
+ * rounds up if the number is > 0.0.
+ */
+#define PROGRESS_ROUNDED(x) (ceil (((x) * 100) - 0.49) / 100)
+
+#define TRACKER_SERVICE "org.freedesktop.Tracker1"
 
 /**
  * SECTION:tracker-miner
@@ -44,18 +57,57 @@
 
 static GQuark miner_error_quark = 0;
 
+/* Introspection data for the service we are exporting */
+static const gchar introspection_xml[] =
+  "<node>"
+  "  <interface name='org.freedesktop.Tracker1.Miner'>"
+  "    <method name='GetStatus'>"
+  "      <arg type='s' name='status' direction='out' />"
+  "    </method>"
+  "    <method name='GetProgress'>"
+  "      <arg type='d' name='progress' direction='out' />"
+  "    </method>"
+  "    <method name='GetPauseDetails'>"
+  "      <arg type='as' name='pause_applications' direction='out' />"
+  "      <arg type='as' name='pause_reasons' direction='out' />"
+  "    </method>"
+  "    <method name='Pause'>"
+  "      <arg type='s' name='application' direction='in' />"
+  "      <arg type='s' name='reason' direction='in' />"
+  "      <arg type='i' name='cookie' direction='out' />"
+  "    </method>"
+  "    <method name='Resume'>"
+  "      <arg type='i' name='cookie' direction='in' />"
+  "    </method>"
+  "    <method name='IgnoreNextUpdate'>"
+  "      <arg type='as' name='urls' direction='in' />"
+  "    </method>"
+  "    <signal name='Started' />"
+  "    <signal name='Stopped' />"
+  "    <signal name='Paused' />"
+  "    <signal name='Resumed' />"
+  "    <signal name='Progress'>"
+  "      <arg type='s' name='status' />"
+  "      <arg type='d' name='progress' />"
+  "    </signal>"
+  "  </interface>"
+  "</node>";
+
 struct _TrackerMinerPrivate {
 	TrackerSparqlConnection *connection;
-
 	GHashTable *pauses;
-
 	gboolean started;
-
 	gchar *name;
 	gchar *status;
 	gdouble progress;
-
 	gint availability_cookie;
+	GDBusConnection *d_connection;
+	GDBusNodeInfo *introspection_data;
+	guint watch_name_id;
+	guint registration_id;
+	guint own_id;
+	gchar *full_name;
+	gchar *full_path;
 };
 
 typedef struct {
@@ -96,9 +148,6 @@ static void       miner_constructed            (GObject       *object);
 static void       pause_data_destroy           (gpointer       data);
 static PauseData *pause_data_new               (const gchar   *application,
                                                 const gchar   *reason);
-static void       store_name_monitor_cb        (TrackerMiner  *miner,
-                                                const gchar   *name,
-                                                gboolean       available);
 
 G_DEFINE_ABSTRACT_TYPE (TrackerMiner, tracker_miner, G_TYPE_OBJECT)
 
@@ -256,7 +305,12 @@ tracker_miner_init (TrackerMiner *miner)
 	miner->private = priv = TRACKER_MINER_GET_PRIVATE (miner);
 
 	priv->connection = tracker_sparql_connection_get (NULL, &error);
-	g_assert_no_error (error);
+
+	/* If we couldn't get a proper SPARQL connection, log error and abort */
+	if (!priv->connection) {
+		g_error ("Couldn't get a proper SPARQL connection (%s), exiting...",
+		         error ? error->message : "Unknown error");
+	}
 
 	priv->pauses = g_hash_table_new_full (g_direct_hash,
 	                                      g_direct_equal,
@@ -270,6 +324,18 @@ miner_update_progress (TrackerMiner *miner)
 	g_signal_emit (miner, signals[PROGRESS], 0,
 	               miner->private->status,
 	               miner->private->progress);
+
+	if (miner->private->d_connection) {
+		g_dbus_connection_emit_signal (miner->private->d_connection,
+		                               NULL,
+		                               miner->private->full_path,
+		                               TRACKER_MINER_DBUS_INTERFACE,
+		                               "Progress",
+		                               g_variant_new ("(sd)",
+		                                              miner->private->status,
+		                                              miner->private->progress),
+		                               NULL);
+	}
 }
 
 static void
@@ -303,10 +369,15 @@ miner_set_property (GObject      *object,
 	case PROP_PROGRESS: {
 		gdouble new_progress;
 
-		new_progress = g_value_get_double (value);
+		new_progress = PROGRESS_ROUNDED (g_value_get_double (value));
 
-		/* Only notify 1% changes */
-		if ((gint) (miner->private->progress * 100) == (gint) (new_progress * 100)) {
+		/* NOTE: We don't round the current progress before
+		 * comparison because we use the rounded value when
+		 * we set it last.
+		 *
+		 * Only notify 1% changes
+		 */ 
+		if (new_progress == miner->private->progress) {
 			/* Same, do nothing */
 			break;
 		}
@@ -345,77 +416,6 @@ miner_get_property (GObject    *object,
 	}
 }
 
-static void
-miner_finalize (GObject *object)
-{
-	TrackerMiner *miner = TRACKER_MINER (object);
-
-	g_free (miner->private->status);
-	g_free (miner->private->name);
-
-	if (miner->private->connection) {
-		g_object_unref (miner->private->connection);
-	}
-
-	g_hash_table_unref (miner->private->pauses);
-
-	_tracker_miner_dbus_remove_name_watch (miner,
-	                                       "org.freedesktop.Tracker1",
-	                                       store_name_monitor_cb);
-	_tracker_miner_dbus_shutdown (miner);
-
-	G_OBJECT_CLASS (tracker_miner_parent_class)->finalize (object);
-}
-
-static void
-miner_constructed (GObject *object)
-{
-	TrackerMiner *miner = TRACKER_MINER (object);
-
-	_tracker_miner_dbus_init (miner, &dbus_glib__tracker_miner_object_info);
-	_tracker_miner_dbus_add_name_watch (miner, 
-	                                    "org.freedesktop.Tracker1",
-	                                    store_name_monitor_cb);
-}
-
-static void
-store_name_monitor_cb (TrackerMiner *miner,
-                       const gchar  *name,
-                       gboolean      available)
-{
-	GError *error = NULL;
-
-	g_debug ("Miner:'%s' noticed store availability has changed to %s",
-	         miner->private->name,
-	         available ? "AVAILABLE" : "UNAVAILABLE");
-
-	if (available && miner->private->availability_cookie != 0) {
-		tracker_miner_resume (miner,
-		                      miner->private->availability_cookie,
-		                      &error);
-
-		if (error) {
-			g_warning ("Error happened resuming miner, %s", error->message);
-			g_error_free (error);
-		}
-
-		miner->private->availability_cookie = 0;
-	} else if (!available && miner->private->availability_cookie == 0) {
-		gint cookie_id;
-
-		cookie_id = tracker_miner_pause (miner,
-		                                 _("Data store is not available"),
-		                                 &error);
-
-		if (error) {
-			g_warning ("Could not pause, %s", error->message);
-			g_error_free (error);
-		} else {
-			miner->private->availability_cookie = cookie_id;
-		}
-	}
-}
-
 static PauseData *
 pause_data_new (const gchar *application,
                 const gchar *reason)
@@ -473,6 +473,16 @@ tracker_miner_start (TrackerMiner *miner)
 	miner->private->started = TRUE;
 
 	g_signal_emit (miner, signals[STARTED], 0);
+
+	if (miner->private->d_connection) {
+		g_dbus_connection_emit_signal (miner->private->d_connection,
+		                               NULL,
+		                               miner->private->full_path,
+		                               TRACKER_MINER_DBUS_INTERFACE,
+		                               "Started",
+		                               NULL,
+		                               NULL);
+	}
 }
 
 /**
@@ -490,6 +500,16 @@ tracker_miner_stop (TrackerMiner *miner)
 	miner->private->started = FALSE;
 
 	g_signal_emit (miner, signals[STOPPED], 0);
+
+	if (miner->private->d_connection) {
+		g_dbus_connection_emit_signal (miner->private->d_connection,
+		                               NULL,
+		                               miner->private->full_path,
+		                               TRACKER_MINER_DBUS_INTERFACE,
+		                               "Stopped",
+		                               NULL,
+		                               NULL);
+	}
 }
 
 /**
@@ -558,6 +578,16 @@ tracker_miner_pause_internal (TrackerMiner  *miner,
 		/* Pause */
 		g_message ("Miner:'%s' is pausing", miner->private->name);
 		g_signal_emit (miner, signals[PAUSED], 0);
+
+		if (miner->private->d_connection) {
+			g_dbus_connection_emit_signal (miner->private->d_connection,
+			                               NULL,
+			                               miner->private->full_path,
+			                               TRACKER_MINER_DBUS_INTERFACE,
+			                               "Paused",
+			                               NULL,
+			                               NULL);
+		}
 	}
 
 	return pd->cookie;
@@ -623,6 +653,16 @@ tracker_miner_resume (TrackerMiner  *miner,
 		/* Resume */
 		g_message ("Miner:'%s' is resuming", miner->private->name);
 		g_signal_emit (miner, signals[RESUMED], 0);
+
+		if (miner->private->d_connection) {
+			g_dbus_connection_emit_signal (miner->private->d_connection,
+			                               NULL,
+			                               miner->private->full_path,
+			                               TRACKER_MINER_DBUS_INTERFACE,
+			                               "Resumed",
+			                               NULL,
+			                               NULL);
+		}
 	}
 
 	return TRUE;
@@ -634,61 +674,147 @@ tracker_miner_get_connection (TrackerMiner *miner)
 	return miner->private->connection;
 }
 
-/* DBus methods */
-void
-_tracker_miner_dbus_get_status (TrackerMiner           *miner,
-                                DBusGMethodInvocation  *context,
-                                GError                **error)
+static void
+miner_finalize (GObject *object)
 {
-	guint request_id;
+	TrackerMiner *miner = TRACKER_MINER (object);
 
-	request_id = tracker_dbus_get_next_request_id ();
+	if (miner->private->watch_name_id != 0) {
+		g_bus_unwatch_name (miner->private->watch_name_id);
+	}
 
-	tracker_dbus_async_return_if_fail (miner != NULL, context);
+	if (miner->private->own_id != 0) {
+		g_bus_unown_name (miner->private->own_id);
+	}
 
-	tracker_dbus_request_new (request_id, context, "%s()", __PRETTY_FUNCTION__);
+	if (miner->private->registration_id != 0) {
+		g_dbus_connection_unregister_object (miner->private->d_connection,
+		                                     miner->private->registration_id);
+	}
 
-	tracker_dbus_request_success (request_id, context);
-	dbus_g_method_return (context, miner->private->status);
+	if (miner->private->introspection_data) {
+		g_dbus_node_info_unref (miner->private->introspection_data);
+	}
+
+	if (miner->private->d_connection) {
+		g_object_unref (miner->private->d_connection);
+	}
+
+	g_free (miner->private->status);
+	g_free (miner->private->name);
+	g_free (miner->private->full_name);
+	g_free (miner->private->full_path);
+
+	if (miner->private->connection) {
+		g_object_unref (miner->private->connection);
+	}
+
+	g_hash_table_unref (miner->private->pauses);
+
+	G_OBJECT_CLASS (tracker_miner_parent_class)->finalize (object);
 }
 
-void
-_tracker_miner_dbus_get_progress (TrackerMiner           *miner,
-                                  DBusGMethodInvocation  *context,
-                                  GError                **error)
+static void
+handle_method_call_ignore_next_update (TrackerMiner          *miner,
+                                       GDBusMethodInvocation *invocation,
+                                       GVariant              *parameters)
 {
-	guint request_id;
+	GStrv urls = NULL;
+	TrackerDBusRequest *request;
 
-	request_id = tracker_dbus_get_next_request_id ();
+	g_variant_get (parameters, "(^a&s)", &urls);
 
-	tracker_dbus_async_return_if_fail (miner != NULL, context);
+	request = tracker_g_dbus_request_begin (invocation,
+	                                        "%s", __PRETTY_FUNCTION__);
 
-	tracker_dbus_request_new (request_id, context, "%s()", __PRETTY_FUNCTION__);
+	tracker_miner_ignore_next_update (miner, urls);
 
-	tracker_dbus_request_success (request_id, context);
-	dbus_g_method_return (context, miner->private->progress);
+	tracker_dbus_request_end (request, NULL);
+	g_dbus_method_invocation_return_value (invocation, NULL);
+	g_free (urls);
 }
 
-void
-_tracker_miner_dbus_get_pause_details (TrackerMiner           *miner,
-                                       DBusGMethodInvocation  *context,
-                                       GError                **error)
+static void
+handle_method_call_resume (TrackerMiner          *miner,
+                           GDBusMethodInvocation *invocation,
+                           GVariant              *parameters)
+{
+	GError *local_error = NULL;
+	gint cookie;
+	TrackerDBusRequest *request;
+
+	g_variant_get (parameters, "(i)", &cookie);
+
+	request = tracker_g_dbus_request_begin (invocation,
+	                                        "%s(cookie:%d)",
+	                                        __PRETTY_FUNCTION__,
+	                                        cookie);
+
+	if (!tracker_miner_resume (miner, cookie, &local_error)) {
+		tracker_dbus_request_end (request, local_error);
+
+		g_dbus_method_invocation_return_gerror (invocation, local_error);
+
+		g_error_free (local_error);
+		return;
+	}
+
+	tracker_dbus_request_end (request, NULL);
+	g_dbus_method_invocation_return_value (invocation, NULL);
+}
+
+static void
+handle_method_call_pause (TrackerMiner          *miner,
+                          GDBusMethodInvocation *invocation,
+                          GVariant              *parameters)
+{
+	GError *local_error = NULL;
+	gint cookie;
+	const gchar *application = NULL, *reason = NULL;
+	TrackerDBusRequest *request;
+
+	g_variant_get (parameters, "(&s&s)", &application, &reason);
+
+	tracker_gdbus_async_return_if_fail (application != NULL, invocation);
+	tracker_gdbus_async_return_if_fail (reason != NULL, invocation);
+
+	request = tracker_g_dbus_request_begin (invocation,
+	                                        "%s(application:'%s', reason:'%s')",
+	                                        __PRETTY_FUNCTION__,
+	                                        application,
+	                                        reason);
+
+	cookie = tracker_miner_pause_internal (miner, application, reason, &local_error);
+	if (cookie == -1) {
+		tracker_dbus_request_end (request, local_error);
+
+		g_dbus_method_invocation_return_gerror (invocation, local_error);
+
+		g_error_free (local_error);
+
+		return;
+	}
+
+	tracker_dbus_request_end (request, NULL);
+	g_dbus_method_invocation_return_value (invocation,
+	                                       g_variant_new ("(i)", cookie));
+}
+
+static void
+handle_method_call_get_pause_details (TrackerMiner          *miner,
+                                      GDBusMethodInvocation *invocation,
+                                      GVariant              *parameters)
 {
 	GSList *applications, *reasons;
 	GStrv applications_strv, reasons_strv;
 	GHashTableIter iter;
 	gpointer key, value;
-	guint request_id;
+	TrackerDBusRequest *request;
 
-	request_id = tracker_dbus_get_next_request_id ();
-
-	tracker_dbus_async_return_if_fail (miner != NULL, context);
-
-	tracker_dbus_request_new (request_id, context, "%s()", __PRETTY_FUNCTION__);
+	request = tracker_g_dbus_request_begin (invocation, "%s()", __PRETTY_FUNCTION__);
 
 	applications = NULL;
 	reasons = NULL;
-
 	g_hash_table_iter_init (&iter, miner->private->pauses);
 	while (g_hash_table_iter_next (&iter, &key, &value)) {
 		PauseData *pd = value;
@@ -696,120 +822,243 @@ _tracker_miner_dbus_get_pause_details (TrackerMiner           *miner,
 		applications = g_slist_prepend (applications, pd->application);
 		reasons = g_slist_prepend (reasons, pd->reason);
 	}
-
 	applications = g_slist_reverse (applications);
 	reasons = g_slist_reverse (reasons);
-
 	applications_strv = tracker_gslist_to_string_list (applications);
 	reasons_strv = tracker_gslist_to_string_list (reasons);
 
-	tracker_dbus_request_success (request_id, context);
-	dbus_g_method_return (context, applications_strv, reasons_strv);
+	tracker_dbus_request_end (request, NULL);
+	g_dbus_method_invocation_return_value (invocation,
+	                                       g_variant_new ("(^as^as)",
+	                                                      applications_strv,
+	                                                      reasons_strv));
 
 	g_strfreev (applications_strv);
 	g_strfreev (reasons_strv);
-
 	g_slist_free (applications);
 	g_slist_free (reasons);
 }
 
-void
-_tracker_miner_dbus_pause (TrackerMiner           *miner,
-                           const gchar            *application,
-                           const gchar            *reason,
-                           DBusGMethodInvocation  *context,
-                           GError                **error)
+static void
+handle_method_call_get_progress (TrackerMiner          *miner,
+                                 GDBusMethodInvocation *invocation,
+                                 GVariant              *parameters)
 {
-	GError *local_error = NULL;
-	guint request_id;
-	gint cookie;
+	TrackerDBusRequest *request;
 
-	request_id = tracker_dbus_get_next_request_id ();
+	request = tracker_g_dbus_request_begin (invocation, "%s()", __PRETTY_FUNCTION__);
 
-	tracker_dbus_async_return_if_fail (miner != NULL, context);
-	tracker_dbus_async_return_if_fail (application != NULL, context);
-	tracker_dbus_async_return_if_fail (reason != NULL, context);
+	tracker_dbus_request_end (request, NULL);
+	g_dbus_method_invocation_return_value (invocation,
+	                                       g_variant_new ("(d)", miner->private->progress));
+}
 
-	tracker_dbus_request_new (request_id, context,
-	                          "%s(application:'%s', reason:'%s')",
-	                          __PRETTY_FUNCTION__,
-	                          application,
-	                          reason);
+static void
+handle_method_call_get_status (TrackerMiner          *miner,
+                               GDBusMethodInvocation *invocation,
+                               GVariant              *parameters)
+{
+	TrackerDBusRequest *request;
 
-	cookie = tracker_miner_pause_internal (miner, application, reason, &local_error);
-	if (cookie == -1) {
-		GError *actual_error = NULL;
+	request = tracker_g_dbus_request_begin (invocation, "%s()", __PRETTY_FUNCTION__);
 
-		tracker_dbus_request_failed (request_id,
-		                             context,
-		                             &actual_error,
-		                             local_error ? local_error->message : NULL);
-		dbus_g_method_return_error (context, actual_error);
+	tracker_dbus_request_end (request, NULL);
+	g_dbus_method_invocation_return_value (invocation,
+	                                       g_variant_new ("(s)", miner->private->status ? miner->private->status : ""));
 
-		g_error_free (actual_error);
-		g_error_free (local_error);
+}
 
+static void
+handle_method_call (GDBusConnection       *connection,
+                    const gchar           *sender,
+                    const gchar           *object_path,
+                    const gchar           *interface_name,
+                    const gchar           *method_name,
+                    GVariant              *parameters,
+                    GDBusMethodInvocation *invocation,
+                    gpointer               user_data)
+{
+	TrackerMiner *miner = user_data;
+
+	tracker_gdbus_async_return_if_fail (miner != NULL, invocation);
+
+	if (g_strcmp0 (method_name, "IgnoreNextUpdate") == 0) {
+		handle_method_call_ignore_next_update (miner, invocation, parameters);
+	} else
+	if (g_strcmp0 (method_name, "Resume") == 0) {
+		handle_method_call_resume (miner, invocation, parameters);
+	} else
+	if (g_strcmp0 (method_name, "Pause") == 0) {
+		handle_method_call_pause (miner, invocation, parameters);
+	} else
+	if (g_strcmp0 (method_name, "GetPauseDetails") == 0) {
+		handle_method_call_get_pause_details (miner, invocation, parameters);
+	} else
+	if (g_strcmp0 (method_name, "GetProgress") == 0) {
+		handle_method_call_get_progress (miner, invocation, parameters);
+	} else
+	if (g_strcmp0 (method_name, "GetStatus") == 0) {
+		handle_method_call_get_status (miner, invocation, parameters);
+	} else {
+		g_assert_not_reached ();
+	}
+}
+
+static GVariant *
+handle_get_property (GDBusConnection  *connection,
+                     const gchar      *sender,
+                     const gchar      *object_path,
+                     const gchar      *interface_name,
+                     const gchar      *property_name,
+                     GError          **error,
+                     gpointer          user_data)
+{
+	g_assert_not_reached ();
+	return NULL;
+}
+
+static gboolean
+handle_set_property (GDBusConnection  *connection,
+                     const gchar      *sender,
+                     const gchar      *object_path,
+                     const gchar      *interface_name,
+                     const gchar      *property_name,
+                     GVariant         *value,
+                     GError          **error,
+                     gpointer          user_data)
+{
+	g_assert_not_reached ();
+	return TRUE;
+}
+
+static void
+on_tracker_store_appeared (GDBusConnection *connection,
+                           const gchar     *name,
+                           const gchar     *name_owner,
+                           gpointer         user_data)
+
+{
+	TrackerMiner *miner = user_data;
+
+	g_debug ("Miner:'%s' noticed store availability has changed to AVAILABLE",
+	         miner->private->name);
+
+	if (miner->private->availability_cookie != 0) {
+		GError *error = NULL;
+
+		tracker_miner_resume (miner,
+		                      miner->private->availability_cookie,
+		                      &error);
+
+		if (error) {
+			g_warning ("Error happened resuming miner, %s", error->message);
+			g_error_free (error);
+		}
+
+		miner->private->availability_cookie = 0;
+	}
+}
+
+static void
+on_tracker_store_disappeared (GDBusConnection *connection,
+                              const gchar     *name,
+                              gpointer         user_data)
+{
+	TrackerMiner *miner = user_data;
+
+	g_debug ("Miner:'%s' noticed store availability has changed to UNAVAILABLE",
+	         miner->private->name);
+
+	if (miner->private->availability_cookie == 0) {
+		GError *error = NULL;
+		gint cookie_id;
+
+		cookie_id = tracker_miner_pause (miner,
+		                                 _("Data store is not available"),
+		                                 &error);
+
+		if (error) {
+			g_warning ("Could not pause, %s", error->message);
+			g_error_free (error);
+		} else {
+			miner->private->availability_cookie = cookie_id;
+		}
+	}
+}
+
+static void
+miner_constructed (GObject *object)
+{
+	TrackerMiner *miner;
+	gchar *name, *full_path, *full_name;
+	GError *error = NULL;
+	GDBusInterfaceVTable interface_vtable = {
+		handle_method_call,
+		handle_get_property,
+		handle_set_property
+	};
+
+	miner = TRACKER_MINER (object);
+	miner->private->d_connection = g_bus_get_sync (G_BUS_TYPE_SESSION, NULL, &error);
+
+	if (!miner->private->d_connection) {
+		g_critical ("Could not connect to the D-Bus session bus, %s",
+		            error ? error->message : "no error given.");
+		g_clear_error (&error);
 		return;
 	}
 
-	tracker_dbus_request_success (request_id, context);
-	dbus_g_method_return (context, cookie);
-}
+	miner->private->introspection_data = g_dbus_node_info_new_for_xml (introspection_xml, NULL);
 
-void
-_tracker_miner_dbus_resume (TrackerMiner           *miner,
-                            gint                    cookie,
-                            DBusGMethodInvocation  *context,
-                            GError                **error)
-{
-	GError *local_error = NULL;
-	guint request_id;
+	g_object_get (miner, "name", &name, NULL);
 
-	request_id = tracker_dbus_get_next_request_id ();
+	if (!name) {
+		g_critical ("Miner '%s' should have been given a name, bailing out",
+		            G_OBJECT_TYPE_NAME (miner));
+		g_assert_not_reached ();
+	}
 
-	tracker_dbus_async_return_if_fail (miner != NULL, context);
+	full_name = g_strconcat (TRACKER_MINER_DBUS_NAME_PREFIX, name, NULL);
 
-	tracker_dbus_request_new (request_id,
-	                          context,
-	                          "%s(cookie:%d)",
-	                          __PRETTY_FUNCTION__,
-	                          cookie);
+	miner->private->own_id = g_bus_own_name_on_connection (miner->private->d_connection,
+	                                                       full_name,
+	                                                       G_BUS_NAME_OWNER_FLAGS_NONE,
+	                                                       NULL, NULL, NULL, NULL);
+	miner->private->full_name = full_name;
 
-	if (!tracker_miner_resume (miner, cookie, &local_error)) {
-		GError *actual_error = NULL;
+	/* Register the service name for the miner */
+	full_path = g_strconcat (TRACKER_MINER_DBUS_PATH_PREFIX, name, NULL);
 
-		tracker_dbus_request_failed (request_id,
-		                             context,
-		                             &actual_error,
-		                             local_error ? local_error->message : NULL);
-		dbus_g_method_return_error (context, actual_error);
+	g_message ("Registering D-Bus object...");
+	g_message ("  Path:'%s'", full_path);
+	g_message ("  Object Type:'%s'", G_OBJECT_TYPE_NAME (miner));
 
-		g_error_free (actual_error);
-		g_error_free (local_error);
+	miner->private->registration_id =
+		g_dbus_connection_register_object (miner->private->d_connection,
+	                                       full_path,
+	                                       miner->private->introspection_data->interfaces[0],
+	                                       &interface_vtable,
+	                                       miner,
+	                                       NULL,
+	                                       &error);
 
+	if (error) {
+		g_critical ("Could not register the D-Bus object %s, %s",
+		            full_path,
+		            error ? error->message : "no error given.");
+		g_clear_error (&error);
 		return;
 	}
 
-	tracker_dbus_request_success (request_id, context);
-	dbus_g_method_return (context);
-}
+	g_free (name);
 
-void
-_tracker_miner_dbus_ignore_next_update (TrackerMiner           *miner,
-                                        const GStrv             urls,
-                                        DBusGMethodInvocation  *context,
-                                        GError                **error)
-{
-	guint request_id;
+	miner->private->full_path = full_path;
 
-	request_id = tracker_dbus_get_next_request_id ();
-
-	tracker_dbus_async_return_if_fail (miner != NULL, context);
-
-	tracker_dbus_request_new (request_id, context, "%s()", __PRETTY_FUNCTION__);
-
-	tracker_miner_ignore_next_update (miner, urls);
-
-	tracker_dbus_request_success (request_id, context);
-	dbus_g_method_return (context);
+	miner->private->watch_name_id = g_bus_watch_name (G_BUS_TYPE_SESSION,
+	                                                  TRACKER_SERVICE,
+	                                                  G_BUS_NAME_WATCHER_FLAGS_NONE,
+	                                                  on_tracker_store_appeared,
+	                                                  on_tracker_store_disappeared,
+	                                                  miner,
+	                                                  NULL);
 }
