@@ -1,18 +1,18 @@
 /*
  * Copyright (C) 2008, Nokia <ivan.frade@nokia.com>
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
- * version 2 of the License, or (at your option) any later version.
+ * version 2.1 of the License, or (at your option) any later version.
  *
- * This program is distributed in the hope that it will be useful,
+ * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * General Public License for more details.
+ * Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU General Public
- * License along with this program; if not, write to the
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, write to the
  * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
  * Boston, MA  02110-1301, USA.
  */
@@ -31,12 +31,9 @@
 #include <glib/gstdio.h>
 #include <gio/gio.h>
 
-#include <dbus/dbus-glib-bindings.h>
-
 #include <libtracker-miner/tracker-miner.h>
 
 #include "tracker-albumart.h"
-#include "tracker-dbus.h"
 #include "tracker-extract.h"
 #include "tracker-marshal.h"
 #include "tracker-albumart-generic.h"
@@ -51,15 +48,17 @@ typedef struct {
 	gchar *local_uri;
 } GetFileInfo;
 
-static void albumart_queue_cb (DBusGProxy     *proxy,
-                               DBusGProxyCall *call,
-                               gpointer        user_data);
 
 static gboolean initialized;
 static gboolean disable_requests;
 static TrackerStorage *albumart_storage;
 static GHashTable *albumart_cache;
-static DBusGProxy *albumart_proxy;
+static GDBusConnection *connection;
+
+static void
+albumart_queue_cb (GObject      *source_object,
+                   GAsyncResult *res,
+                   gpointer      user_data);
 
 static gboolean
 albumart_strip_find_next_block (const gchar    *original,
@@ -293,14 +292,16 @@ albumart_get_path (const gchar  *artist,
 		}
 
 		parent = g_file_get_parent (file);
-		local_dir = g_file_get_uri (parent);
+		if (parent) {
+			local_dir = g_file_get_uri (parent);
 
-		/* This is a URI, don't use g_build_filename here */
-		*local_uri = g_strdup_printf ("%s/.mediaartlocal/%s", local_dir, art_filename);
+			/* This is a URI, don't use g_build_filename here */
+			*local_uri = g_strdup_printf ("%s/.mediaartlocal/%s", local_dir, art_filename);
 
-		g_free (local_dir);
+			g_free (local_dir);
+			g_object_unref (parent);
+		}
 		g_object_unref (file);
-		g_object_unref (parent);
 	}
 
 	g_free (dir);
@@ -320,7 +321,7 @@ albumart_heuristic (const gchar *artist,
 	GDir *dir;
 	GError *error = NULL;
 	gchar *target = NULL;
-	gchar *dirname;
+	gchar *dirname = NULL;
 	const gchar *name;
 	gboolean retval;
 	gint count;
@@ -380,9 +381,12 @@ albumart_heuristic (const gchar *artist,
 
 	file = g_file_new_for_uri (filename_uri);
 	dirf = g_file_get_parent (file);
-	dirname = g_file_get_path (dirf);
+	if (dirf) {
+		dirname = g_file_get_path (dirf);
+		g_object_unref (dirf);
+	}
 	g_object_unref (file);
-	g_object_unref (dirf);
+
 
 	if (!dirname) {
 		g_debug ("Album art directory could not be used:'%s'", dirname);
@@ -461,10 +465,9 @@ albumart_heuristic (const gchar *artist,
 					found_file = g_file_new_for_path (found);
 					g_free (found);
 
-					g_file_copy (found_file, file, 0, NULL, NULL, NULL, &error);
+					retval = g_file_copy (found_file, file, 0, NULL, NULL, NULL, &error);
 					g_object_unref (found_file);
 
-					retval = error != NULL;
 					g_clear_error (&error);
 				}
 			} else if (g_str_has_suffix (name_strdown, "png")) {
@@ -545,45 +548,33 @@ albumart_request_download (TrackerStorage *storage,
                            const gchar    *local_uri,
                            const gchar    *art_path)
 {
-	GetFileInfo *info;
+	if (connection) {
+		GetFileInfo *info;
 
-	if (disable_requests) {
-		return;
-	}
-
-	info = g_slice_new (GetFileInfo);
-
-	info->storage = storage ? g_object_ref (storage) : NULL;
-
-	info->local_uri = g_strdup (local_uri);
-	info->art_path = g_strdup (art_path);
-
-	if (!albumart_proxy) {
-		GError          *error = NULL;
-		DBusGConnection *connection;
-
-		connection = dbus_g_bus_get (DBUS_BUS_SESSION, &error);
-
-		if (!error) {
-			albumart_proxy = dbus_g_proxy_new_for_name (connection,
-			                                            ALBUMARTER_SERVICE,
-			                                            ALBUMARTER_PATH,
-			                                            ALBUMARTER_INTERFACE);
-		} else {
-			g_error_free (error);
+		if (disable_requests) {
+			return;
 		}
-	}
 
-	dbus_g_proxy_begin_call (albumart_proxy,
-	                         "Queue",
-	                         albumart_queue_cb,
-	                         info,
-	                         NULL,
-	                         G_TYPE_STRING, artist,
-	                         G_TYPE_STRING, album,
-	                         G_TYPE_STRING, "album",
-	                         G_TYPE_UINT, 0,
-	                         G_TYPE_INVALID);
+		info = g_slice_new (GetFileInfo);
+
+		info->storage = storage ? g_object_ref (storage) : NULL;
+
+		info->local_uri = g_strdup (local_uri);
+		info->art_path = g_strdup (art_path);
+
+		g_dbus_connection_call (connection,
+		                        ALBUMARTER_SERVICE,
+		                        ALBUMARTER_PATH,
+		                        ALBUMARTER_INTERFACE,
+		                        "Queue",
+		                        g_variant_new ("(sssu)", artist, album, "album", 0),
+		                        NULL,
+		                        G_DBUS_CALL_FLAGS_NONE,
+		                        -1,
+		                        NULL,
+		                        albumart_queue_cb,
+		                        info);
+	}
 }
 
 static void
@@ -635,8 +626,14 @@ albumart_copy_to_local (TrackerStorage *storage,
 			GFile *dirf;
 
 			dirf = g_file_get_parent (local_file);
-			g_file_make_directory_with_parents (dirf, NULL, NULL);
-			g_object_unref (dirf);
+			if (dirf) {
+				/* Parent file may not exist, as if the file is in the
+				 * root of a gvfs mount. In this case we won't try to
+				 * create the parent directory, just try to copy the
+				 * file there. */
+				g_file_make_directory_with_parents (dirf, NULL, NULL);
+				g_object_unref (dirf);
+			}
 
 			g_debug ("Copying album art from:'%s' to:'%s'",
 			         filename, local_uri);
@@ -651,30 +648,31 @@ albumart_copy_to_local (TrackerStorage *storage,
 }
 
 static void
-albumart_queue_cb (DBusGProxy     *proxy,
-                   DBusGProxyCall *call,
-                   gpointer        user_data)
+albumart_queue_cb (GObject      *source_object,
+                   GAsyncResult *res,
+                   gpointer      user_data)
 {
-	GError      *error = NULL;
-	guint        handle;
+	GError *error = NULL;
 	GetFileInfo *info;
+	GVariant *v;
 
 	info = user_data;
 
-	dbus_g_proxy_end_call (proxy, call, &error,
-	                       G_TYPE_UINT, &handle,
-	                       G_TYPE_INVALID);
+	v = g_dbus_connection_call_finish ((GDBusConnection *) source_object, res, &error);
 
 	if (error) {
-		if (error->code == DBUS_GERROR_SERVICE_UNKNOWN) {
+		if (error->code == G_DBUS_ERROR_SERVICE_UNKNOWN) {
 			disable_requests = TRUE;
 		} else {
 			g_warning ("%s", error->message);
 		}
-
 		g_clear_error (&error);
 	}
 
+	if (v) {
+		g_variant_unref (v);
+	}
+
 	if (info->storage && info->art_path &&
 	    g_file_test (info->art_path, G_FILE_TEST_EXISTS)) {
 
@@ -696,7 +694,6 @@ albumart_queue_cb (DBusGProxy     *proxy,
 gboolean
 tracker_albumart_init (void)
 {
-	DBusGConnection *connection;
 	GError *error = NULL;
 
 	g_return_val_if_fail (initialized == FALSE, FALSE);
@@ -710,7 +707,7 @@ tracker_albumart_init (void)
 	                                        NULL);
 
 	/* Signal handler for new album art from the extractor */
-	connection = dbus_g_bus_get (DBUS_BUS_SESSION, &error);
+	connection = g_bus_get_sync (G_BUS_TYPE_SESSION, NULL, &error);
 
 	if (!connection) {
 		g_critical ("Could not connect to the D-Bus session bus, %s",
@@ -719,12 +716,6 @@ tracker_albumart_init (void)
 		return FALSE;
 	}
 
-	/* Get album art downloader proxy */
-	albumart_proxy = dbus_g_proxy_new_for_name (connection,
-	                                            ALBUMARTER_SERVICE,
-	                                            ALBUMARTER_PATH,
-	                                            ALBUMARTER_INTERFACE);
-
 	initialized = TRUE;
 
 	return TRUE;
@@ -735,8 +726,8 @@ tracker_albumart_shutdown (void)
 {
 	g_return_if_fail (initialized == TRUE);
 
-	if (albumart_proxy) {
-		g_object_unref (albumart_proxy);
+	if (connection) {
+		g_object_unref (connection);
 	}
 
 	if (albumart_cache) {
@@ -804,14 +795,16 @@ tracker_albumart_process (const unsigned char *buffer,
 		} else {
 			/* If not, we perform a heuristic on the dir */
 			gchar *key;
-			gchar *dirname;
+			gchar *dirname = NULL;
 			GFile *file, *dirf;
 
 			file = g_file_new_for_uri (filename_uri);
 			dirf = g_file_get_parent (file);
-			dirname = g_file_get_path (dirf);
+			if (dirf) {
+				dirname = g_file_get_path (dirf);
+				g_object_unref (dirf);
+			}
 			g_object_unref (file);
-			g_object_unref (dirf);
 
 			key = g_strdup_printf ("%s-%s-%s",
 			                       artist ? artist : "",
